// Copyright 2007 by Intuit Inc.
// All rights reserved
// Usage governed by the QuickBooks SDK Developer's License Agreement

using System;
using System.Collections.Generic;
using System.Text;
using Interop.QBFC10;
using Intuit_QBFC1.Session_Framework;

namespace Intuit_QBFC1.Subscription_Framework.Data
{
    class DataSubscriptionManager
    {
        private SessionManager sessionMgr = SessionManager.getInstance();               // Instance holder
        private ISubscriptionMsgSetRequest _msgSet = null;                              // Holder for the subscription message set
        private IDataEventSubscriptionAdd _request = null;                              // Holder for the request object
        private Guid _guid = Guid.Empty;                                                // Holder for the subscriber GUID; initially empty
        private Guid _classId;                                                          // Holder for the COM classID
        private string _progId;                                                         // Holder for the COM progID
        private ENDeliveryPolicy _deliveryPolicy = ENDeliveryPolicy.dpDeliverAlways;    // Event Delivery Policy
        private ENTrackLostEvents _trackLostEvents = ENTrackLostEvents.tleNone;         // Event Track Lost Events Policy
        private bool _deliveryOwnEvents = true;                                         // Events Deliver Own Events Policy

        // Don't allow the default constructor...
        private DataSubscriptionManager() { }

        public DataSubscriptionManager(Guid subscriberId, Guid classId)
            : this(subscriberId, null)
        {
            _classId = classId;
        }

        public DataSubscriptionManager(Guid subscriberId, string progId)
        {
            _guid = subscriberId;
            _progId = progId;

            try
            {
                // go and determine if the backend is even capable of processing subscriptions...
                if (sessionMgr.QBsdkVersion < 3.0)
                {
                    Logger.getInstance().logError("DataSubscriptionManager.DataSubscriptionManager", "Backend is too old to support subscriptions!");
                    throw new DataSubscriptionException("Opps! Your backend does not support subscriptions!");
                }

                // Next determine if the GUID is there...
                if (_guid == Guid.Empty)
                {
                    Logger.getInstance().logError("DataSubscriptionManager.DataSubscriptionManager", "A GUID for the application was not provided and is mandatory");
                    throw new DataSubscriptionException("Opps! No GUID for the application was found!");
                }
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.DataSubscriptionManager", e.Message);
                throw e;
            }
        }

        /// <summary>
        /// Sets up the request objects...
        /// </summary>
        /// <returns>the data event subscription object</returns>
        private IDataEventSubscriptionAdd getSubscriptionAddRq()
        {
            if (_msgSet == null)
            {
                _msgSet = sessionMgr.getSubscriptionMsgSetRequest();
                _request = null;
            }

            if (_request == null)
                _request = _msgSet.AppendDataEventSubscriptionAddRq();

            // Setup the request...
            _request.SubscriberID.SetValue(_guid.ToString("B"));
            _request.COMCallbackInfo.AppName.SetValue(sessionMgr.AppName);

            // Set the ProgID or ClassID, as appropriate
            if (_classId != Guid.Empty)
                _request.COMCallbackInfo.ORProgCLSID.CLSID.SetValue(_classId.ToString("B"));
            else if (_progId != null)
                _request.COMCallbackInfo.ORProgCLSID.ProgID.SetValue(_progId.ToString());
            else
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.getSubscriptionAddRq", "Either the COM classID or COM progID must be provided!");
                throw new DataSubscriptionException("Error: Either the COM classID or COM progID must be provided!");
            }

            return _request;
        }

        /// <summary>
        /// Clean-up method that should always be called when done
        /// </summary>
        private void cleanUpSubscriptionAddRq()
        {
            _request = null;
            _msgSet = null;
        }

        /// <summary>
        /// Deletes any data event subscriptions for this application in QuickBooks
        /// </summary>
        public void deleteDataSubscriptions()
        {
            try
            {
                // Create a message request set.
                _msgSet = sessionMgr.getSubscriptionMsgSetRequest();

                // get the query object, set the GUID and then execute the request
                ISubscriptionDel lQuery = _msgSet.AppendSubscriptionDelRq();
                lQuery.SubscriberID.SetValue(_guid.ToString("B"));
                lQuery.SubscriptionType.SetValue(ENSubscriptionType.stData);
                sessionMgr.doSubscriptionRequests(ref _msgSet);
                _msgSet = null;
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.deleteDataSubscriptions", e.Message);
                throw e;
            }
        }

        /// <summary>
        /// Gets/Sets the event's delivery policy (see the OSR)
        /// </summary>
        public ENDeliveryPolicy DeliveryPolicy
        {
            get
            {
                return _deliveryPolicy;
            }
            set
            {
                _deliveryPolicy = value;
            }
        }

        /// <summary>
        /// Gets/Sets the policy on tracking lost events (see the OSR)
        /// </summary>
        public ENTrackLostEvents TrackLostEvents
        {
            get
            {
                return _trackLostEvents;
            }
            set
            {
                _trackLostEvents = value;
            }
        }

        /// <summary>
        /// Gets/Sets the policy on delivery of your own application sourced events (see the OSR)
        /// </summary>
        public bool DeliveryOwnEvents
        {
            get
            {
                return _deliveryOwnEvents;
            }
            set
            {
                _deliveryOwnEvents = value;
            }
        }

        /// <summary>
        /// Saves a subscription, which must be subsequently submitted to QuickBooks through subscribe().
        /// The method is smart enough that it can handle either list or transaction subscription types
        /// </summary>
        /// <param name="eventDef">the event definition, either a ListEvent or TxnEvent object</param>
        public void addDataSubscription(IEventBase eventDef)
        {
            if (eventDef is ListEvent)
                addDataListSubscription(((ListEvent)eventDef).EventType, ((ListEvent)eventDef).EventOperation);
            else
                addDataTxnSubscription(((TxnEvent)eventDef).EventType, ((TxnEvent)eventDef).EventOperation);
        }

        /// <summary>
        /// Saves a List Event subscription, which must be subsequently submitted to QuickBooks through 
        /// subscribe(). 
        /// </summary>
        /// <param name="eventType">The List Event's types</param>
        /// <param name="eventOp">The List Event's operations</param>
        public void addDataListSubscription(ENListEventType[] eventType, ENListEventOperation[] eventOp)
        {
            if (eventType.Length == 0 || eventOp.Length == 0)
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.addDataListSubscription", "you must have at least one value in both Event Type and Event Operation");
                throw new DataSubscriptionException("Data Subscription Error: you must have at least one value in both Event Type and Event Operation");
            }

            getSubscriptionAddRq();
            IListEventSubscription listEvent = _request.ListEventSubscriptionList.Append();

            foreach (ENListEventType obj in eventType)
                listEvent.ListEventTypeList.Add(obj);
            foreach (ENListEventOperation obj in eventOp)
                listEvent.ListEventOperationList.Add(obj);
        }

        /// <summary>
        /// Saves a Transaction Event subscription, which must be subsequently submitted to QuickBooks through 
        /// subscribe(). 
        /// </summary>
        /// <param name="eventType">The Transaction Event's types</param>
        /// <param name="eventOp">The Transaction Event's operations</param>
        public void addDataTxnSubscription(ENTxnEventType[] eventType, ENTxnEventOperation[] eventOp)
        {
            if (eventType.Length == 0 || eventOp.Length == 0)
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.addDataTxnSubscription", "you must have at least one value in both Event Type and Event Operation");
                throw new DataSubscriptionException("Data Subscription Error: you must have at least one value in both Event Type and Event Operation");
            }

            getSubscriptionAddRq();
            ITxnEventSubscription txnEvent = _request.TxnEventSubscriptionList.Append();

            foreach (ENTxnEventType obj in eventType)
                txnEvent.TxnEventTypeList.Add(obj);
            foreach (ENTxnEventOperation obj in eventOp)
                txnEvent.TxnEventOperationList.Add(obj);
        }

        /// <summary>
        /// Subscribes the previously setup data events
        /// </summary>
        public void subscribe()
        {
            try
            {
                if (_msgSet == null || _request == null)
                {
                    Logger.getInstance().logCritical("DataSubscriptionManager.addDataTxnSubscription", "you must have at least one value in both Event Type and Event Operation");
                    throw new Exception("Error: you must setup data events before attempting to subscribe!");
                }

                // Add the request's global event information now
                _request.TrackLostEvents.SetValue(_trackLostEvents);
                _request.DeliveryPolicy.SetValue(_deliveryPolicy);
                _request.DeliverOwnEvents.SetValue(_deliveryOwnEvents);

                // And now subscribe!
                if (sessionMgr.doSubscriptionRequests(ref _msgSet))
                    throw new DataSubscriptionException("Error encountered while attempting to perform Data Subscription!");

                // Always cleanup after yourself!
                cleanUpSubscriptionAddRq();
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("DataSubscriptionManager.subscribe", e.Message);
                throw e;
            }
        }

    }





}
