// Copyright 2007 by Intuit Inc.
// All rights reserved
// Usage governed by the QuickBooks SDK Developer's License Agreement

using System;
using System.Collections;
using Interop.QBFC10;
using Intuit_QBFC1.Session_Framework;

namespace Intuit_QBFC1.Subscription_Framework.Menu
{
    public class MenuSubscriptionManager
    {
        private SessionManager sessionMgr = SessionManager.getInstance();   // Instance holder
        private ISubscriptionMsgSetRequest _menuSet = null;                 // Holder for the subscription message set
        private Guid _guid = Guid.Empty;                                    // Holder for the subscriber GUID; initially empty
        private Guid _classId;                                              // Holder for the COM classID
        private string _progId;                                             // Holder for the COM progID

        // Don't allow the default constructor...
        private MenuSubscriptionManager() { }

        public MenuSubscriptionManager(Guid subscriberId, Guid classId)
            : this(subscriberId, null)
        {
            _classId = classId;
        }

        public MenuSubscriptionManager(Guid subscriberId, string progId)
        {
            _guid = subscriberId;
            _progId = progId;

            try
            {
                // go and determine if the backend is even capable of processing subscriptions...
                if (sessionMgr.QBsdkVersion < 3.0)
                {
                    Logger.getInstance().logError("MenuSubscriptionManager.MenuSubscriptionManager", "Backend is too old to support subscriptions!");
                    throw new MenuSubscriptionException("Opps! Your backend does not support subscriptions!");
                }

                // Next determine if the GUID is there...
                if (_guid == Guid.Empty)
                {
                    Logger.getInstance().logError("MenuSubscriptionManager.MenuSubscriptionManager", "A GUID for the application was not provided and is mandatory");
                    throw new MenuSubscriptionException("Opps! No GUID for the application was found!");
                }

                // Next, setup the menu set...
                clearMenuSet();

            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("MenuSubscriptionManager.MenuSubscriptionManager", e.Message);
                throw e;
            }
        }

        /// <summary>
        /// Acquires the menu subscription list and saves it into a class variable
        /// </summary>
        /// <returns>The list of menu subscriptions</returns>
        public IResponseList getMenuSubscriptionList()
        {
            try
            {
                // Create a new menu request set.  The class level one is used for managing *only*
                // the creation of menu items.
                ISubscriptionMsgSetRequest menuSet = sessionMgr.getSubscriptionMsgSetRequest();

                // get the query object, set the GUID and then execute the request
                IUIExtensionSubscriptionQuery lQuery = menuSet.AppendUIExtensionSubscriptionQueryRq();
                lQuery.SubscriberID.SetValue(_guid.ToString("B"));
                sessionMgr.doSubscriptionRequests(ref menuSet);

                // Save the menu list for subsequent usage...
                IResponseList menuList = sessionMgr.getSubscriptionResponseList();

                if (menuList == null)
                {
                    Logger.getInstance().logCritical("MenuSubscriptionManager.getMenuSubscriptionList", "Unable to retrieve the menu subscription list (null)");
                    throw new MenuSubscriptionException("Error: Unable to retrieve the menu subscription list");
                }

                return menuList;
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("MenuSubscriptionManager.getMenuSubscriptionList", e.Message);
                throw e;
            }
        }

        /// <summary>
        /// Deletes any menu subscriptions for this application in QuickBooks
        /// </summary>
        public void deleteMenuSubscriptions()
        {
            try
            {
                // An error will be thrown if you attempt to delete a subscription when one does not
                // presently exist.  Therefore, before deleting the subscription, query Quickbooks to
                // see if a subscription with the same ID already exists.
                IResponseList menuList = getMenuSubscriptionList();

                if (menuList.GetAt(0).StatusCode != 0)
                    return;

                // Create a new menu request set.  The class level one is used for managing *only*
                // the creation of menu items.
                ISubscriptionMsgSetRequest menuSet = sessionMgr.getSubscriptionMsgSetRequest();

                // get the query object, set the GUID and then execute the request
                ISubscriptionDel lQuery = menuSet.AppendSubscriptionDelRq();
                lQuery.SubscriberID.SetValue(_guid.ToString("B"));
                lQuery.SubscriptionType.SetValue(ENSubscriptionType.stUIExtension);
                sessionMgr.doSubscriptionRequests(ref menuSet);
                // menuList = sessionMgr.getSubscriptionResponseList();
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("MenuSubscriptionManager.deleteMenuSubscriptions", e.Message);
                throw e;
            }
        }

        /// <summary>
        /// Used for setting up a menu item.  Subscriptions are atomic; therefore, all menu items 
        /// must be configured prior to executing subscribe method.
        /// </summary>
        /// <param name="menuName">The menu in which the new menu items are to be added</param>
        /// <param name="menu">The menu item definition</param>
        /// <returns></returns>
        public bool addMenuItem(ENAddToMenu menuName, IMenuBuilder menu)
        {
            if (menu == null)
            {
                Logger.getInstance().logError("MenuSubscriptionManager.addMenuItem", "You must supply some form of menu object!");
                throw new MenuSubscriptionException("Error: You must supply some form of menu object!");
            }

            // Create the query...
            IUIExtensionSubscriptionAdd query = _menuSet.AppendUIExtensionSubscriptionAddRq();
            query.SubscriberID.SetValue(_guid.ToString("B"));
            query.COMCallbackInfo.AppName.SetValue(sessionMgr.AppName);

            // Set the ProgID or ClassID, as appropriate
            if (_classId != Guid.Empty)
                query.COMCallbackInfo.ORProgCLSID.CLSID.SetValue(_classId.ToString("B"));
            else if (_progId != null)
                query.COMCallbackInfo.ORProgCLSID.ProgID.SetValue(_progId.ToString());
            else
            {
                Logger.getInstance().logError("MenuSubscriptionManager.addMenuItem", "Either the COM classID or COM progID must be provided!");
                throw new MenuSubscriptionException("Error: Either the COM classID or COM progID must be provided!");
            }

            // Now specify the place where the menu item is to be added
            query.MenuExtensionSubscription.AddToMenu.SetValue(menuName);

            // Finally add the menu item
            if (menu is MenuItemType)
            {
                // Because of the use of properties, the following cannot be moved into a separate 
                // method.  Therefore, go through the effort of creating the menu item here...
                MenuItemType menuIt = (MenuItemType)menu;
                query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.MenuText.SetValue(menuIt.MenuText);
                query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.EventTag.SetValue(menuIt.EventTag);

                for (int i = 0; i < menuIt.VisibleIf.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.DisplayCondition.VisibleIfList.Add(menuIt.VisibleIf[i]);

                for (int i = 0; i < menuIt.VisibleIfNot.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.DisplayCondition.VisibleIfNotList.Add(menuIt.VisibleIfNot[i]);

                for (int i = 0; i < menuIt.EnabledIf.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.DisplayCondition.EnabledIfList.Add(menuIt.EnabledIf[i]);

                for (int i = 0; i < menuIt.EnabledIfNot.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.MenuItem.DisplayCondition.EnabledIfNotList.Add(menuIt.EnabledIfNot[i]);
            }
            else
            {
                SubMenuType menuIt = (SubMenuType)menu;

                // First the SubMenu display conditions...
                for (int i = 0; i < menuIt.VisibleIf.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.Submenu.DisplayCondition.VisibleIfList.Add(menuIt.VisibleIf[i]);

                for (int i = 0; i < menuIt.VisibleIfNot.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.Submenu.DisplayCondition.VisibleIfNotList.Add(menuIt.VisibleIfNot[i]);

                for (int i = 0; i < menuIt.EnabledIf.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.Submenu.DisplayCondition.EnabledIfList.Add(menuIt.EnabledIf[i]);

                for (int i = 0; i < menuIt.EnabledIfNot.Length; ++i)
                    query.MenuExtensionSubscription.ORMenuSubmenu.Submenu.DisplayCondition.EnabledIfNotList.Add(menuIt.EnabledIfNot[i]);

                // now the embedded menu data...
                for (int j = 0; j < menuIt.MenuItems.Length; ++j)
                {
                    IMenuItem embeddedMenu = query.MenuExtensionSubscription.ORMenuSubmenu.Submenu.MenuItemList.Append();
                    embeddedMenu.MenuText.SetValue(menuIt.MenuItems[j].MenuText);
                    embeddedMenu.EventTag.SetValue(menuIt.MenuItems[j].EventTag);

                    for (int i = 0; i < menuIt.MenuItems[j].VisibleIf.Length; ++i)
                        embeddedMenu.DisplayCondition.VisibleIfList.Add(menuIt.MenuItems[j].VisibleIf[i]);

                    for (int i = 0; i < menuIt.MenuItems[j].VisibleIfNot.Length; ++i)
                        embeddedMenu.DisplayCondition.VisibleIfNotList.Add(menuIt.MenuItems[j].VisibleIfNot[i]);

                    for (int i = 0; i < menuIt.MenuItems[j].EnabledIf.Length; ++i)
                        embeddedMenu.DisplayCondition.EnabledIfList.Add(menuIt.MenuItems[j].EnabledIf[i]);

                    for (int i = 0; i < menuIt.MenuItems[j].EnabledIfNot.Length; ++i)
                        embeddedMenu.DisplayCondition.EnabledIfNotList.Add(menuIt.MenuItems[j].EnabledIfNot[i]);
                }
            }

            return true;
        }

        public void subscribe()
        {
            try
            {
                if (_menuSet == null)
                {
                    Logger.getInstance().logCritical("MenuSubscriptionManager.subscribe", "You must setup menus before attempting to subscribe!");
                    throw new MenuSubscriptionException("Error: you must setup menus before attempting to subscribe!");
                }

                if (sessionMgr.doSubscriptionRequests(ref _menuSet))
                    throw new MenuSubscriptionException("Error encountered while attempting to perform Menu Subscription!");

                clearMenuSet();
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("MenuSubscriptionManager.subscribe", e.Message);
                throw e;
            }
        }

        public void clearMenuSet()
        {
            try
            {
                _menuSet = sessionMgr.getSubscriptionMsgSetRequest();
            }
            catch (Exception e)
            {
                Logger.getInstance().logCritical("MenuSubscriptionManager.clearMenuSet", e.Message);
                throw e;
            }
        }
    }
}
